# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-07-17 03:17
from __future__ import unicode_literals

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('cms', '0016_auto_20160608_1535'),
        ('filer', '0010_auto_20180414_2058'),
        ('test_app', '0002_auto_20161128_1458'),
    ]

    operations = [
        migrations.CreateModel(
            name='CMSFilerLinkModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('link_type', models.CharField(blank=True, default='', max_length=20, verbose_name='Link type')),
                ('link_style', models.CharField(blank=True, default='', max_length=64, verbose_name='Link Style')),
                ('free', models.CharField(blank=True, default='', max_length=512, verbose_name='Internal / Other')),
                ('external', models.URLField(blank=True, default='', verbose_name='External Address')),
                ('mailto', models.EmailField(blank=True, default='', max_length=254, verbose_name='E-Mail')),
                ('phone', models.CharField(blank=True, default='', max_length=64, verbose_name='Phone')),
                ('cms_page', cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='cms.Page')),
                ('file', filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='filer.File')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ContribLinkModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('link_type', models.CharField(blank=True, default='', max_length=20, verbose_name='Link type')),
                ('link_style', models.CharField(blank=True, default='', max_length=64, verbose_name='Link Style')),
                ('free', models.CharField(blank=True, default='', max_length=512, verbose_name='Internal / Other')),
                ('external', models.URLField(blank=True, default='', verbose_name='External Address')),
                ('mailto', models.EmailField(blank=True, default='', max_length=254, verbose_name='E-Mail')),
                ('phone', models.CharField(blank=True, default='', max_length=64, verbose_name='Phone')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]
