# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2016-11-28 14:58
from __future__ import unicode_literals

import ckeditor.fields
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='testmodel',
            name='richtext_second',
            field=ckeditor.fields.RichTextField(default=''),
        ),
        migrations.AlterField(
            model_name='testmodel',
            name='richtext',
            field=ckeditor.fields.RichTextField(default=''),
        ),
    ]
