
/* everything !important :( */


/* override normal django styles styles */
.popup #content {
    padding: 5px 13px 13px !important;
    box-sizing: border-box;
}

@media (max-width: 767px) {
    .aligned .form-row > div {
        /* from django itself */
        width: 100% !important;
    }
}

/* override djangocms styles */
.popup #container {
    padding: 0 !important;
}
