# Generated by Django 2.2.15 on 2021-04-12 07:50

import cms.models.fields
from django.db import migrations, models
import django.db.models.deletion
import filer.fields.file


class Migration(migrations.Migration):

    dependencies = [
        ('test_app', '0003_cmsfilerlinkmodel_contriblinkmodel'),
    ]

    operations = [
        migrations.AddField(
            model_name='cmsfilerlinkmodel',
            name='html_anchor',
            field=models.SlugField(blank=True, default='', verbose_name='Anker'),
        ),
        migrations.AlterField(
            model_name='cmsfilerlinkmodel',
            name='cms_page',
            field=cms.models.fields.PageField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='test_app_cmsfilerlinkmodel_set', to='cms.Page'),
        ),
        migrations.AlterField(
            model_name='cmsfilerlinkmodel',
            name='file',
            field=filer.fields.file.FilerFileField(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='test_app_cmsfilerlinkmodel_set', to='filer.File'),
        ),
    ]
